<?php

namespace Bixo\Recomentation\Providers;

use Illuminate\Support\ServiceProvider;

class RecomentationServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'recomentation');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'recomentation');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerRecomentation();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerRecomentation()
    {
        $this->app->bind('recomentation', function($app) {
            return new Recomentation($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Recomentation', 'Lavalite\Recomentation\Facades\Recomentation');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('bixo.recomentation', function ($app) {
            return $this->app->make('Bixo\Recomentation\Recomentation');
        });

                // Bind Recomentation to repository
        $this->app->bind(
            'Bixo\Recomentation\Interfaces\RecomentationRepositoryInterface',
            \Bixo\Recomentation\Repositories\Eloquent\RecomentationRepository::class
        );

        $this->app->register(\Bixo\Recomentation\Providers\AuthServiceProvider::class);
                $this->app->register(\Bixo\Recomentation\Providers\EventServiceProvider::class);
        
        $this->app->register(\Bixo\Recomentation\Providers\RouteServiceProvider::class);
                // $this->app->register(\Bixo\Recomentation\Providers\WorkflowServiceProvider::class);
            }

    /**
     * Merges user's and recomentation's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'bixo.recomentation'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeRecomentation::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['bixo.recomentation'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('bixo/recomentation.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/recomentation')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/recomentation')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
